#!/usr/bin/env python

"""A module providing nice sqlite-based data-aware widgets"""

import qt
import mx.DateTime as mdt

########################################################################
#
# Helper functions to get back values from widgets. These get assigned
# to widget.__value and are used by saveRecord
#
########################################################################

def QDateEditToString(self):
    return '%04d%02d%02d'%(self.date().year(),self.date().month(),self.date().day())

def QTimeEditToString(self):
    return '%02d:%02d:%02d'%(self.time().hour(),self.time().minute(),self.time().second())

def QDateTimeEditToString(self):
    dt=self.dateTime()
    d=dt.date()
    t=dt.time()
    return '%04d%02d%02d %02d:%02d:%02d'%(d.year(),d.month(),d.day(),t.hour(),t.minute(),t.second())

def TextToString(self):
    return str(self.text())

class QPSForm(qt.QObject):
    def __init__(self,database,table):
        qt.QObject.__init__(self)
        self.db=database
        self.table=table
        self.widgets={}
        self.result=None
        self.index=0
        self.primKey=None
        
    def addWidget(self,widget,field):
        """Associates a certain widget with a certain field in the table"""
        self.widgets[widget.name()]=widget
        widget.__field=field
        
    def setQuery(self,query=None,primKey='rowid'):
        """setQuery takes a "WHERE" as argument. For example, if you want 
        the record with ID 1, all you have to send is 'ID=1'.
        
        primKey should contain a primary key, it is used to figure out
        how to do updates and deletes, so please be careful ;-)
        """
        
        self.primKey=primKey
        
        fields=[self.widgets[x].__field for x in self.widgets]
        if query:
            cmd='SELECT %s,%s FROM %s WHERE %s;'%(primKey,','.join(fields),self.table,query)
        else:
            cmd='SELECT %s,%s FROM %s;'%(primKey,','.join(fields),self.table)
            
        print cmd
        
        self.cu=self.db.cursor()
        self.cu.execute(cmd)
        self.result=self.cu.fetchall()
        self.index=0
        
    def setRecord(self,index):
        """Makes the form display the nth record in the result set"""
        #Sanity checks
        
        if not self.result:
            return
        
        if 0>index: #too low
            return
        
        elif index>=len(self.result): #too high
            return
        
        if 0==index: #at start
            self.emit (qt.PYSIGNAL('atStart'),(True,))
        else:
            self.emit (qt.PYSIGNAL('atStart'),(False,))
        
        if index+1==len(self.result):
            self.emit (qt.PYSIGNAL('atEnd'),(True,))
        else:
            self.emit (qt.PYSIGNAL('atEnd'),(False,))
        
        self.index=index
        res=self.result[index]
        print "new index ",index
        
        print res
        
        if res:
            for widget in self.widgets:
                self.key=res[self.primKey]
                self.setValue(self.widgets[widget],res[self.widgets[widget].__field])
        else:
            #FIXME think about it
            pass

    def next(self):
        self.setRecord (self.index+1)
    
    def prev(self):
        self.setRecord (self.index-1)
        
    def setValue(self,widget,value):
        """The tricky piece. We examine the widget, the value, and try to fit them"""
        
        # These widgets only take strings and all have 
        # a setText method, so there's no point in finesse
        if (isinstance (widget,qt.QLabel) or
            isinstance (widget,qt.QLineEdit) or
            isinstance (widget,qt.QTextEdit) or
            isinstance (widget,qt.QTextBrowser)):
            
            widget.setText(str(value))
            widget.__value=TextToString
            
        elif (isinstance (widget,qt.QCheckBox) or
            isinstance (widget,qt.QRadioButton)):
            widget.__value=widget.isChecked
            #Hopefully, we are trying to put an integer in it
            if isinstance (value,int) or isinstance (value,long) or isinstance (value,float):
                widget.setChecked(value)
            else:
                #Maybe it can be coerced
                try:
                    widget.setChecked(int(value))
                except ValueError:
                    #Maybe it can't
                    #FIXME Do something
                    pass

        elif isinstance (widget,qt.QDateEdit):
            if isinstance (value,mdt.DateTimeType):
                widget.setDate(qt.QDate(value.year,value.month,value.day))
                widget.__value=QDateEditToString
            else:
                #FIXME do something
                pass
        
        elif isinstance (widget,qt.QTimeEdit):
            if isinstance (value,mdt.DateTimeType):
                widget.setTime(qt.QTime(value.year,value.month,value.day))
                widget.__value=QTimeEditToString
            else:
                #FIXME do something
                pass
        
        elif isinstance (widget,qt.QDateTimeEdit):
            if isinstance (value,mdt.DateTimeType):
                widget.setDateTime(QDateTime(qt.QDate(value.year,value.month,value.day),
                                                qt.QTime(value.hour,value.minute,value.second)))
                widget.__value=QDateTimeEditToString
            else:
                #FIXME do something
                pass

            
    def saveRecord(self):
        data=[]
        for wname in self.widgets:
            data.append(self.widgets[wname].__field+"='%s'"%self.widgets[wname].__value(self.widgets[wname]))
        
        v=','.join(data)

        cmd='UPDATE %s SET %s WHERE %s=%s'%(self.table,v,self.primKey,self.key)
        print cmd
        self.cu.execute(cmd)
        
        # Update the result set's current result from the DB
        
        fields=[self.widgets[x].__field for x in self.widgets]
        cmd="SELECT %s,%s FROM %s WHERE %s='%s';"%(self.primKey,','.join(fields),self.table,self.primKey,self.key)
        print cmd
        self.cu.execute(cmd)
        self.result[self.index]=self.cu.fetchone()
        print self.result[self.index]
        self.db.commit()
                    
if __name__ == "__main__":
    import sys
    import form
    import sqlite

    #A simple single-form Qt app
    
    a = qt.QApplication(sys.argv)
    qt.QObject.connect(a,qt.SIGNAL("lastWindowClosed()"),a,qt.SLOT("quit()"))
    w = form.Form()
    a.setMainWidget(w)
     
    #Create a sample on-memory DB
    
    sampledb="""
    CREATE TABLE sample (
    NAME varchar(60) not null,
    BIRTH DATE not null);
    INSERT INTO sample VALUES('joe','1/1/1971');
    INSERT INTO sample VALUES('jack','1/1/1972');
    INSERT INTO sample VALUES('jane','1/2/1971');
    """    
    
    db=sqlite.connect(':memory:')
#    db=sqlite.connect('sample.db')
    cu=db.cursor()
    cu.execute(sampledb)
    
    #Link the DB's table to the widgetry
    
    psf=QPSForm(db,'sample')
    psf.addWidget(w.name,'name')
    psf.addWidget(w.birth,'birth')
    
    #Connect a few signals so it browses the records nicely
    
    qt.QObject.connect(w.next,qt.SIGNAL('clicked()'),psf.next)
    qt.QObject.connect(w.prev,qt.SIGNAL('clicked()'),psf.prev)
    qt.QObject.connect(w.save,qt.SIGNAL('clicked()'),psf.saveRecord)
        
    qt.QObject.connect(psf,qt.PYSIGNAL('atStart'),w.prev.setDisabled)
    qt.QObject.connect(psf,qt.PYSIGNAL('atEnd'),w.next.setDisabled)
    
    # Initialize the QPSForm so it has something to show
    
    psf.setQuery()
    psf.setRecord(0)
    
    #Go into the event loop
    
    w.show()
    a.exec_loop()
    
    db.close()
    
    
    
